{% extends "!layout.html" %}
{%- block relbar1 %}
<div style="background-color: #E7F0F3; text-align: left; padding: 8px 4px 8px 4px">
  <font size=+3><a href="{{ pathto('index') }}">
      <img src="{{pathto("_static/pyepics.png", 1) }}" height=50 border="0" alt="pyepics"/>
    &nbsp; PyEpics: Epics Channel Access for Python</a>
  </font>
</div>
{{ super() }}
{% endblock %}
