<h3>Download</h3>

<ul>
  <li>Current version: <b>{{ release }}</b></li>
  <li>Install: <tt> pip install pyepics</tt></li>
   <li>Develop: <a href="https://github.com/pyepics/pyepics/">github.com</a></li>
 </ul>


<p>
<h3>Contents</h3>

<ul>
   <li>&nbsp; &nbsp;<a href="{{ pathto('installation') }}"> Install</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('overview') }}"> Overview</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('pv') }}">    Epics PVs</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('ca') }}">    CA module</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('arrays') }}"> Working with Array Data</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('devices') }}"> Devices</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('alarm') }}"> Alarms</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('autosave') }}"> Autosave</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('wx') }}"> wx widgets for PVs</a></li>
   <li>&nbsp; &nbsp;<a href="{{ pathto('advanced') }}"> Advanced Topics</a></li>
</ul>
